//==============================================================================
//
// Title:      IviSpecAn_ReadTraceAndMarkers_Example 
// Purpose:    Demonstration example on how to use rsscope IVI-C driver
//             to read the trace and set the marker.
// Notes:      To run the example you need to install rsscope IVI-C driver 2.0 or later.
//
//             Default locations of instrument driver files on 32bit Windows:
//             "c:\Program Files (x86)\IVI Foundation\IVI\Lib\msc\ivispecan.lib"
//             "c:\Program Files (x86)\IVI Foundation\IVI\Include\IviSpecAn.h" 
//             "c:\Program Files (x86)\IVI Foundation\IVI\Drivers\rsspecan" 
//
//             Default locations of instrument driver files on 64bit Windows:
//             "c:\Program Files\IVI Foundation\IVI\Lib_x64\msc\ivispecan.lib"
//             "c:\Program Files\IVI Foundation\IVI\Include\ivispecan.h"
//             "c:\Program Files\IVI Foundation\IVI\Drivers\rsspecan"
//
// Created on: 01/2017 by Miloslav Macko.
// Copyright:  Rohde & Schwarz GmbH & Co. KG. All Rights Reserved.


//==============================================================================
// Include files
#include <stdio.h>
#include <windows.h>
#include "IviSpecAn.h"

//==============================================================================
// Constants
#define STRING_LENGTH 4096  

//==============================================================================
// Global variables
ViSession g_io = 0;
static ViString g_sResourceName = "IviCspecAnFSW";

//==============================================================================
// Global functions
/// HIFN Checks for instrument error
/// HIRET Error code
/// HIPAR status/Status code to be checked, returned from instrument driver function call
ViStatus checkError (ViStatus status)
{
    ViChar  error_message [STRING_LENGTH];
    ViChar  error_buffer  [STRING_LENGTH * 2];
    ViChar* p2buf;
    ViInt32 error;
    
    if (status < VI_SUCCESS)
    {
        IviSpecAn_error_message (g_io, status, error_message);
        p2buf = error_buffer + sprintf (error_buffer, "Primary Error: 0x%08X, %s\n", status, error_message);
        
        
        /* This function reads an error code and a message from the instrument's error queue */
        IviSpecAn_error_query (g_io, &error, error_message);
        
        if (error != 0)
        {
            sprintf (p2buf, "Secondary Error: Instrument is reporting the following error (SYST:ERR?): %d, %s\n", error, error_message);
        }
        
        IviSpecAn_close (g_io);
    printf ("\nERROR!\n\n");
    printf (error_buffer);
    printf ("\nPress ENTER to continue...");
    (void) getchar();
        
    }
    return status;
}

//==============================================================================
// Macros

#define CHECKERR(fCal) \
    if (status = checkError((fCal)), status < VI_SUCCESS) \
        return status; else

#define TRACE_BUFFER_LENGTH 4096
//==============================================================================
int main (int argc, char *argv[])
{
    ViStatus status = VI_SUCCESS;
    ViReal64 x, y;
    ViInt32 acqStatus = 0, actualPoints = -1;
    ViReal64 yTrace[TRACE_BUFFER_LENGTH];
    
    printf ("Initializing instrument '%s' ... ", g_sResourceName);
    CHECKERR ( IviSpecAn_init ("IviCspecAnFSW", VI_TRUE, VI_TRUE, &g_io) );
    printf ("success.\n\n");
    
    CHECKERR ( IviSpecAn_ConfigureFrequencyCenterSpan(g_io, 2.456E9, 1.234E9) );
    CHECKERR ( IviSpecAn_ConfigureSweepCoupling (g_io, VI_FALSE, 100E3, VI_FALSE, 50E3, VI_FALSE, 4.5) );
    CHECKERR ( IviSpecAn_SetActiveMarker(g_io, "MARKER1") );
    CHECKERR ( IviSpecAn_ConfigureMarkerEnabled(g_io, VI_TRUE, "TR1") );
    CHECKERR ( IviSpecAn_SetActiveMarker(g_io, "MARKER16") );
    CHECKERR ( IviSpecAn_ConfigureMarkerEnabled(g_io, VI_TRUE, "TR1") );
    
    // Start the measurement. This function does not wait for the acquisition to be completed.
    // It needsa to be done externally with polling the IviSpecAn_AcquisitionStatus()
    CHECKERR ( IviSpecAn_Initiate(g_io) );

    //Synchronization for IviSpecAn_Initiate - waiting for the measurement to complete
    printf ("Acquisition Process: ");
    do
    {
        CHECKERR ( IviSpecAn_AcquisitionStatus(g_io, &acqStatus) );
        Sleep(100);
        printf (".");
    } while (acqStatus != IVISPECAN_VAL_ACQUISITION_STATUS_COMPLETE);
    printf (" finished\n");
    
    // Configure which difference in amplitute should be considered a peak
    CHECKERR ( IviSpecAn_ConfigureMarkerSearch(g_io, 1.0, -90) );
    
    // Marker 1 to Max
    CHECKERR ( IviSpecAn_SetActiveMarker(g_io, "MARKER1") );
    CHECKERR ( IviSpecAn_MarkerSearch (g_io, IVISPECAN_VAL_MARKER_SEARCH_HIGHEST) );    
    CHECKERR ( IviSpecAn_QueryMarker(g_io, &x, &y) );
    printf ("Marker 1 Peak: %0.1fHz: %0.2fdBm\n", x, y);
    
    // Marker 16 to Next Left
    CHECKERR ( IviSpecAn_SetActiveMarker(g_io, "MARKER16") );
    CHECKERR ( IviSpecAn_MarkerSearch (g_io, IVISPECAN_VAL_MARKER_SEARCH_HIGHEST) );    
    CHECKERR ( IviSpecAn_MarkerSearch (g_io, IVISPECAN_VAL_MARKER_SEARCH_NEXT_PEAK_LEFT) );
    CHECKERR ( IviSpecAn_QueryMarker(g_io, &x, &y) );
    printf ("Marker 16 Next Left: %0.1fHz: %0.2f2fdBm\n", x, y);
    
    // IviSpecAn_ReadYTrace does waiting for the measurement to complete already inside, no need to do it outside
    printf ("\nReading the Trace with ReadYTrace...");
    CHECKERR ( IviSpecAn_ReadYTrace (g_io, "TRACE1", 7000, TRACE_BUFFER_LENGTH, &actualPoints, yTrace) );
    printf ("success.\n");
    
    // Marker 1 to Max
    CHECKERR ( IviSpecAn_SetActiveMarker(g_io, "MARKER1") );
    CHECKERR ( IviSpecAn_MarkerSearch (g_io, IVISPECAN_VAL_MARKER_SEARCH_HIGHEST) );
    CHECKERR ( IviSpecAn_QueryMarker(g_io, &x, &y) );
    printf ("Marker 1 Peak: %0.1fHz: %0.2f2fdBm\n", x, y);
    
    // Marker 16 to Min
    CHECKERR ( IviSpecAn_SetActiveMarker(g_io, "MARKER16") );
    CHECKERR ( IviSpecAn_MarkerSearch (g_io, IVISPECAN_VAL_MARKER_SEARCH_MINIMUM) );
    CHECKERR ( IviSpecAn_QueryMarker(g_io, &x, &y) );
    printf ("Marker 16 Min: %0.1fHz: %0.2f2fdBm\n", x, y);
    
    // Marker 1 OFF
    CHECKERR ( IviSpecAn_SetActiveMarker(g_io, "MARKER1") );
    CHECKERR ( IviSpecAn_ConfigureMarkerEnabled (g_io, VI_FALSE, "") );    
    
    // Marker 16 OFF
    CHECKERR ( IviSpecAn_SetActiveMarker(g_io, "MARKER16") );
    CHECKERR ( IviSpecAn_ConfigureMarkerEnabled (g_io, VI_FALSE, "") );
    
    printf ("\nChecking overall error....\n");
    checkError(status);
    printf ("No errors\ occured.\n");
    
    printf ("\nClosing the session to the instrument '%s' ... ", g_sResourceName);
    CHECKERR ( IviSpecAn_close(g_io) );
    printf ("success.\n\n");

    printf ("\nPress ENTER to continue...");
    (void) getchar();
    
    return 0;
}

